﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=6.47
@EndOfDesignText@
'xRotaryKnob CustomView Class
'
'Written by Klaus CHRISTL (klaus)
'
'Version: 1.4 2020.01.19
'Amended CustomKnobFileName property repved variable type Boolean by String
'
'Version: 1.3 2019.04.09
'Added Visible property
'
'Version: 1.2 2019.04.08
'Added Value property, allowing to preset a position
'
'Version: 1.1 2019.01.27
'Added CustomKnob and CustomKnobFileName properties with a given Bitmap
'
'Version: 1.0 2018.09.08
'
'Events declaration
#Event: ValueChanged(Value As Int)
#RaisesSynchronousEvents: ValueChanged

'Designer property declarations
#DesignerProperty: Key: OffsetAngle, DisplayName: OffsetAngle, FieldType: Int, DefaultValue: 45, Description: Offset angle of the first scale value from the bottom
#DesignerProperty: Key: ScaleNbValues, DisplayName: ScaleNbValues, FieldType: Int, DefaultValue: 10, Description: Number of scale values
#DesignerProperty: Key: ScaleMinValue, DisplayName: ScaleMinValue, FieldType: Int, DefaultValue: 1, Description: Min valiue of the scale
#DesignerProperty: Key: ScaleMaxValue, DisplayName: ScaleMaxValue, FieldType: Int, DefaultValue: 10, Description: Max valiue of the scale
#DesignerProperty: Key: BackgroundColor, DisplayName: BackgroundColor, FieldType: Color, DefaultValue: 0xFFFFFFFF, Description: Knob color
#DesignerProperty: Key: SnapMode, DisplayName: SnapMode, FieldType: String, DefaultValue: AFTERMOVE, List: NEVER|AFTERMOVE|ALLWAYS, Description: Snap mode
#DesignerProperty: Key: KnobColor, DisplayName: KnobColor, FieldType: Color, DefaultValue: 0xFFFF0000, Description: Background color
#DesignerProperty: Key: KnobBorderColor, DisplayName: BorderColor, FieldType: Color, DefaultValue: 0xFF000000, Description: Knob border color
#DesignerProperty: Key: LineColor, DisplayName: LineColor, FieldType: Color, DefaultValue: 0xFFFFFFFF, Description: Knob line color
#DesignerProperty: Key: TextColor, DisplayName: TextColor, FieldType: Color, DefaultValue: 0xFF000000, Description: Text color
#DesignerProperty: Key: HighlightTextColor, DisplayName: HighlightTextColor, FieldType: Color, DefaultValue: 0xFFFF0000, Description: Text color fo highlighted scale values
#DesignerProperty: Key: CustomKnob, DisplayName: CustomKnob, FieldType: Boolean, DefaultValue: False, Description: CustomKnob with a Bitmap. The file must be in File.DirAsset
#DesignerProperty: Key: CustomKnobFileName, DisplayName: CustomKnobFileName, FieldType: String, DefaultValue: , Description: CustomKnob Bitmap file name. Valid only with CustomKnob = True. The file must be in File.DirAsset

Sub Class_Globals
	Private mEventName As String 'ignore
	Private mCallBack As Object 'ignore
	Private xui As XUI 'ignore
	
	Private xBase, xParent As B4XView
	Private xcvsBase As B4XCanvas
	Private xTextFont As B4XFont
	Private xbmpKnob As B4XBitmap
	Private xRectBase, xRectKnob, xRectValue As B4XRect
	Private xPathKnob As B4XPath
	
	Private mLeft, mTop, mWidth, mHeight As Double
	
	Private mCenterX, mCenterY, mRadius As Int
	Private mTextSize As Float
	Private mTextHeight, mTextValueY As Int
	
	Private mBackgroundColor, mTextColor, mHighlightTextColor, mKnobColor, mKnobBorderColor, mLineColor As Int
	Private mScaleNbValues, mScaleMinValue, mScaleMaxValue, mScaleDivision As Int
	Private mValue, mIndex As Int
	Private mAngle, mCurrentAngle, mOffsetAngle, mDivisionAngle, mBeginAngle, mEndAngle As Double
	Private TouchRadius, mScale As Double
	Private mSnapMode As String
	Private mLineWidth As Int
	
	Private tmrClick As Timer
	Private mTimeDown, mClickTime As Long
	Private mCustomKnob As Boolean
	Private mCustomKnobFileName As String
End Sub

Public Sub Initialize (Callback As Object, EventName As String)
	mEventName = EventName
	mCallBack = Callback
	
	mOffsetAngle = 45
	mScaleNbValues = 10
	mScaleMinValue = 1
	mScaleMaxValue = 10
	mScaleDivision = (mScaleMaxValue - mScaleMinValue) / (mScaleNbValues - 1)
	mSnapMode = "AFTERMOVE"
	
	mBackgroundColor = xui.Color_White
	mKnobColor = xui.Color_Red
	mKnobBorderColor = xui.Color_Black
	mLineColor = xui.Color_White
	mTextColor = xui.Color_Black
	mHighlightTextColor = xui.Color_Red
	
	mClickTime = 400
	tmrClick.Initialize("tmrClick", mClickTime)
	mCustomKnob = False
End Sub

'Base type must be Object
Public Sub DesignerCreateView (Base As Object, Lbl As Label, Props As Map)
	Private xDummy As B4XView
	
	xDummy = Base
	mLeft = xDummy.Left
	mTop = xDummy.Top
	mWidth = Max(xDummy.Width, xDummy.Height)
	mHeight = mWidth
	
	mCenterX = mWidth / 2
	mCenterY = mCenterX
	
	xParent = xDummy.Parent
	xDummy.RemoveViewFromParent

	mOffsetAngle = Props.Get("OffsetAngle")
	mScaleNbValues = Props.Get("ScaleNbValues")
	mScaleMinValue = Props.Get("ScaleMinValue")
	mScaleMaxValue = Props.Get("ScaleMaxValue")
	mScaleDivision = (mScaleMaxValue - mScaleMinValue) / (mScaleNbValues - 1)
	mSnapMode = Props.Get("SnapMode")
	mCustomKnob = Props.Get("CustomKnob")
	
	mBackgroundColor = xui.PaintOrColorToColor(Props.Get("BackgroundColor"))
	mKnobColor = xui.PaintOrColorToColor(Props.Get("KnobColor"))
	mKnobBorderColor = xui.PaintOrColorToColor(Props.Get("KnobBorderColor"))
	mLineColor = xui.PaintOrColorToColor(Props.Get("LineColor"))
	mTextColor = xui.PaintOrColorToColor(Props.Get("TextColor"))
	mHighlightTextColor = xui.PaintOrColorToColor(Props.Get("HighlightTextColor"))
	If mCustomKnob = True Then
		mCustomKnobFileName = Props.Get("CustomKnobFileName")
	End If
	
	InitClass
End Sub

Private Sub Base_Resize (Width As Double, Height As Double)
	mWidth = Max(Width, Height)
	mHeight = mWidth
	
	InitClass
End Sub

Public Sub AddToParent(Parent As B4XView, Left As Double, Top As Double, Width As Double, Height As Double)
	mLeft = Left
	mTop = Top
	mWidth = Max(Width, Height)
	mHeight = mWidth
	
	mCenterX = mWidth / 2
	mCenterY = mCenterX
	
	xParent = Parent
	
	InitClass
End Sub

Private Sub InitClass
	xBase = xui.CreatePanel("xBase")
	xBase.Tag = Me		'allows to use Parent.GetView(x).Tag
	xParent.AddView(xBase, mLeft, mTop, mWidth, mHeight)
	
	xcvsBase.Initialize(xBase)
	xRectBase.Initialize(0, 0, mWidth, mHeight)
	mRadius = 0.3 * mWidth
	xRectKnob.Initialize(mCenterX - mRadius, mCenterY - mRadius, mCenterX + mRadius, mCenterY + mRadius)
	Private mBaseWidth As Int
	mBaseWidth = 200 * xui.Scale
	If mWidth / mBaseWidth > 1 Then
		mTextSize = 15 * (1 + 0.5 * (mWidth / mBaseWidth - 1))
	Else
		mTextSize = 15 * (1 +  (mWidth / mBaseWidth - 1))
	End If
	xTextFont = xui.CreateDefaultBoldFont(mTextSize)
	mTextHeight = MeasureTextHeight("0", xTextFont)
	
	xRectValue.Initialize(0, mCenterY + mRadius + 5dip, mWidth, mHeight)
	mTextValueY	= (mHeight + mCenterY + mRadius) / 2 + mTextHeight
	If mWidth / xui.Scale > 200 Then
		mLineWidth = 3dip
	Else
		mLineWidth = 2dip
	End If
	
	mBeginAngle = -270 + mOffsetAngle
	mEndAngle = 90 - mOffsetAngle
	mDivisionAngle = (360 - 2 * mOffsetAngle) / (mScaleNbValues - 1)
	mScale = (mScaleMaxValue - mScaleMinValue) / (360 - 2 * mOffsetAngle)
	
	InitKnobBitmap
	DrawScale
	DrawKnob(mBeginAngle)
	If mSnapMode <> "NEVER" Then
		DrawScaleDigit(mIndex, mHighlightTextColor)
	Else
		mValue = mScaleMinValue - (mBeginAngle - mBeginAngle) * mScale
		DrawValue(mValue)
	End If
End Sub

Private Sub xBase_Touch (Action As Int, X As Float, Y As Float)
	Private Index As Int
	
	Select Action
		Case xBase.TOUCH_ACTION_DOWN
			mTimeDown = DateTime.Now
			TouchRadius = Sqrt((X - mCenterX) * (X - mCenterX) + (Y - mCenterY) * (Y - mCenterY))
			mAngle = AdjustAngle(ATan2D((Y - mCenterY), (X - mCenterX)))
			If TouchRadius > mRadius / 2 Then
				DrawKnob(mAngle)
			Else
				tmrClick.Enabled = True
			End If
		Case xBase.TOUCH_ACTION_MOVE
			If TouchRadius > mRadius / 2 Then
				mAngle = AdjustAngle(ATan2D((Y - mCenterY), (X - mCenterX)))
				Index = GetIndexAngle(mAngle)
				If Index <> mIndex Then
					DrawScaleDigit(mIndex, mTextColor)
					mIndex = Index
					If mSnapMode <> "NEVER" Then
						DrawScaleDigit(mIndex, mHighlightTextColor)					
					End If
				End If
				Select mSnapMode
					Case "NEVER"
						DrawKnob(mAngle)
						mValue = mScaleMinValue - (mBeginAngle - mAngle) * mScale
						DrawValue(mValue)
					Case "AFTERMOVE"
						DrawKnob(mAngle)
					Case "ALLWAYS"
						mCurrentAngle = mBeginAngle + mIndex * mDivisionAngle
						DrawKnob(mCurrentAngle)
				End Select
			End If
		Case xBase.TOUCH_ACTION_UP
			If TouchRadius > mRadius / 2 Then
				DrawScaleDigit(mIndex, mTextColor)
				mAngle = AdjustAngle(ATan2D((Y - mCenterY), (X - mCenterX)))
				mIndex = GetIndexAngle(mAngle)
				mCurrentAngle = mBeginAngle + mIndex * mDivisionAngle
				Select mSnapMode
					Case "NEVER"
						DrawKnob(mAngle)
						mValue = mScaleMinValue + (mAngle - mBeginAngle) * mScale
					Case "AFTERMOVE", "ALLWAYS"
						mValue = mScaleMinValue + mIndex * mScaleDivision
						DrawKnob(mCurrentAngle)
						DrawScaleDigit(mIndex, mHighlightTextColor)
				End Select
				CallValueChanged
			Else
				If DateTime.Now - mTimeDown <= mClickTime Then
					DrawScaleDigit(mIndex, mTextColor)
					mIndex = mIndex + 1
					mIndex = Min(mIndex, (mScaleNbValues - 1))
					mValue = mScaleMinValue + mIndex * mScaleDivision
					mCurrentAngle = mBeginAngle + mIndex * mDivisionAngle
					tmrClick.Enabled = False
					DrawScaleDigit(mIndex, mHighlightTextColor)
					DrawKnob(mCurrentAngle)
					CallValueChanged
				End If
			End If
	End Select
End Sub

Private Sub tmrClick_Tick
	DrawScaleDigit(mIndex, mTextColor)
	mIndex = mIndex - 1
	mIndex = Max(mIndex, 0)
	mValue = mScaleMinValue + mIndex * mScaleDivision
	mCurrentAngle = mBeginAngle + mIndex * mDivisionAngle
	DrawScaleDigit(mIndex, mHighlightTextColor)
	
	tmrClick.Enabled = False
	DrawKnob(mCurrentAngle)
	CallValueChanged
End Sub

Private Sub CallValueChanged
	If xui.SubExists(mCallBack, mEventName & "_ValueChanged", 1) Then
		CallSubDelayed2(mCallBack, mEventName & "_ValueChanged", mValue)
	End If
End Sub

Private Sub InitKnobBitmap
	xPathKnob.InitializeOval(xRectKnob)
	If mCustomKnob = True Then
		xcvsBase.ClearRect(xRectBase)
		xcvsBase.ClipPath(xPathKnob)
		xcvsBase.DrawBitmap(xui.LoadBitmapResize(File.DirAssets, mCustomKnobFileName, xRectKnob.Width, xRectKnob.Height, True), xRectKnob)
'		xbmpKnob = xui.LoadBitmapResize(File.DirAssets, mCustomKnobFileName, xRectKnob.Width, xRectKnob.Height, True)
	Else
	
		xcvsBase.ClearRect(xRectBase)
		xcvsBase.ClipPath(xPathKnob)
		xcvsBase.DrawCircle(mCenterX, mCenterY, mRadius, mKnobColor, True, 1dip)
		xcvsBase.DrawLine(mCenterX + 0.2 * mRadius, mCenterY, mCenterX + mRadius, mCenterY, mLineColor, mLineWidth + 1dip)
		xcvsBase.DrawLine(mCenterX - mRadius, mCenterY - 0.2 * mRadius, mCenterX + mRadius, mCenterY - 0.2 * mRadius, mKnobBorderColor, mLineWidth)
		xcvsBase.DrawLine(mCenterX - mRadius, mCenterY + 0.2 * mRadius, mCenterX + mRadius, mCenterY + 0.2 * mRadius, mKnobBorderColor, mLineWidth)
		xcvsBase.DrawCircle(mCenterX, mCenterY, mRadius - 1dip, mKnobBorderColor, False, mLineWidth)
		xcvsBase.Invalidate
	End If

#If B4A
	xbmpKnob = xBase.Snapshot
	#Else
		xbmpKnob = xcvsBase.CreateBitmap
#End If
		xcvsBase.RemoveClip
		xcvsBase.DrawRect(xRectBase, mBackgroundColor, True, 1dip)
End Sub

Private Sub AdjustAngle(Angle As Double) As Double
	If Angle > 90 Then Angle = Angle - 360
	Angle = Max(Angle, mBeginAngle)
	Angle = Min(Angle, mEndAngle)
	Return Angle
End Sub

Private Sub GetIndexAngle(Angle As Double) As Int
	Private Index As Int
	
	Index = Abs(Floor((mBeginAngle - Angle) / mDivisionAngle + 0.5))
	Return Index
End Sub

Private Sub GetIndexValue(Value As Int) As Int
	Private Index As Int
	
	Index = Abs(Floor((Value - mScaleMinValue) / (mScaleNbValues - 1) + 0.5))
	Return Index
End Sub

Private Sub DrawScale
	Private Index, x1, y1, x2, y2 As Int
	Private Angle, Radius1, Radius2 As Double
	
	For Index = 0 To mScaleNbValues - 1
		Angle = mBeginAngle + Index * mDivisionAngle
		Radius1 = 1.02 * mRadius
		Radius2 = 1.15 * mRadius
		x1 = mCenterX + Radius1 * CosD(Angle)
		y1 = mCenterY + Radius1 * SinD(Angle)
		x2 = mCenterX + Radius2 * CosD(Angle)
		y2 = mCenterY + Radius2 * SinD(Angle)
		
		xcvsBase.DrawLine(x1, y1, x2, y2, mTextColor, mLineWidth)
		DrawScaleDigit(Index, mTextColor)
	Next
	xcvsBase.Invalidate
End Sub

Private Sub DrawKnob(Angle As Double)
	xcvsBase.DrawBitmapRotated(xbmpKnob, xRectBase, Angle)
	xcvsBase.Invalidate
End Sub

Private Sub DrawScaleDigit(Index As Int, Color As Int)
	Private x1, y1, DeltaX, DeltaY As Int
	Private Angle, Radius As Double
	Private txt As String
	
	txt = mScaleMinValue + Index * mScaleDivision
	Angle = mBeginAngle + Index * mDivisionAngle
	Radius = 1.25 * mRadius 
	
	DeltaX = MeasureTextWidth(txt, xTextFont) / 2 * CosD(Angle)
	DeltaY = MeasureTextHeight(txt, xTextFont) / 2 * SinD(Angle)
	x1 = mCenterX + Radius * CosD(Angle) + DeltaX
	y1 = mCenterY + Radius * SinD(Angle) + 0.5 * mTextHeight + DeltaY
	xcvsBase.DrawText(txt, x1, y1, xTextFont, Color, "CENTER")

	xcvsBase.Invalidate
End Sub

'Draws the current value, only in the "NEVER" snap mode
Private Sub DrawValue(Value As Int)
	Private txt As String	

	txt = NumberFormat(Value, 1, 0)
	xcvsBase.DrawRect(xRectValue, mBackgroundColor, True, 1dip)
	xcvsBase.DrawText(txt, mCenterX, mTextValueY, xTextFont, mTextColor, "CENTER")
End Sub

'Draws the whole object
Private Sub DrawAll
	InitKnobBitmap
	DrawScale
	DrawKnob(mBeginAngle)
	DrawScaleDigit(mIndex, mHighlightTextColor)
End Sub

'gets or sets the Left property
Public Sub getLeft As Double
	Return mLeft
End Sub

Public Sub setLeft(Left As Double)
	mLeft = Left
	xBase.Left = mLeft
End Sub

'gets or sets the Top property
Public Sub getTop As Double
	Return mTop
End Sub

Public Sub setTop(Top As Double)
	mTop  = Top 
	xBase.Top  = mTop
End Sub

'gets or sets the Width property
'there is no Height property because the object is square
Public Sub getWidth As Double
	Return mWidth
End Sub

Public Sub setWidth(Width As Double)
	mWidth  = Width
	mHeight = mWidth
	xBase.Width  = mWidth
	xcvsBase.Resize(mWidth, mHeight)
	If xBase.IsInitialized Then
		DrawAll
	End If
End Sub

'gets or sets the OffsetAngle property
Public Sub getOffsetAngle As Int
	Return mOffsetAngle
End Sub

Public Sub setOffsetAngle(OffsetAngle As Int)
	mOffsetAngle = OffsetAngle
	mBeginAngle = -270 + OffsetAngle
	mEndAngle = 90 - mOffsetAngle
	mDivisionAngle = (360 - 2 * mOffsetAngle) / (mScaleNbValues - 1)
	mScale = (mScaleMaxValue - mScaleMinValue) / (360 - 2 * mOffsetAngle)
	If xBase.IsInitialized Then
		DrawAll
	End If
End Sub

'gets or sets the ScaleNbValues property
Public Sub getScaleNbValues As Int
	Return mScaleNbValues
End Sub

Public Sub setScaleNbValues(ScaleNbValues As Int)
	mScaleNbValues = ScaleNbValues
	mScaleDivision = (mScaleMaxValue - mScaleMinValue) / (mScaleNbValues - 1)
End Sub

'gets or sets the ScaleMinValue property
Public Sub getScaleMinValue As Int
	Return mScaleMinValue
End Sub

Public Sub setScaleMinValue(ScaleMinValue As Int)
	mScaleMinValue = ScaleMinValue
	mScaleDivision = (mScaleMaxValue - mScaleMinValue) / (mScaleNbValues - 1)
	mScale = (mScaleMaxValue - mScaleMinValue) / (360 - 2 * mOffsetAngle)
	If xBase.IsInitialized Then
		DrawAll
	End If
End Sub

'gets or sets the ScaleMaxValue property
Public Sub getScaleMaxValue As Int
	Return mScaleMaxValue
End Sub

Public Sub setScaleMaxValue(ScaleMaxValue As Int)
	mScaleMaxValue = ScaleMaxValue
	mScaleDivision = (mScaleMaxValue - mScaleMinValue) / (mScaleNbValues - 1)
	mScale = (mScaleMaxValue - mScaleMinValue) / (360 - 2 * mOffsetAngle)
	If xBase.IsInitialized Then
		DrawAll
	End If
End Sub

'gets or sets the BackgroundColor property
Public Sub getBackgroundColor As Int
	Return mBackgroundColor
End Sub

Public Sub setBackgroundColor(BackgroundColor As Int)
	mBackgroundColor = BackgroundColor
	If xBase.IsInitialized Then
		DrawAll
	End If
End Sub

'gets or sets the KnobColor property
Public Sub getKnobColor As Int
	Return mKnobColor
End Sub

Public Sub setKnobColor(KnobColor As Int)
	mKnobColor = KnobColor
	If xBase.IsInitialized Then
		DrawAll
	End If
End Sub

'gets or sets the KnobBorderColor property
Public Sub getKnobBorderColor As Int
	Return mKnobBorderColor
End Sub

Public Sub setKnobBorderColor(KnobBorderColor As Int)
	mKnobBorderColor = KnobBorderColor
	If xBase.IsInitialized Then
		DrawAll
	End If
End Sub

'gets or sets the LineColor property
Public Sub getLineColor As Int
	Return mLineColor
End Sub

Public Sub setLineColor(LineColor As Int)
	mLineColor = LineColor
	If xBase.IsInitialized Then
		DrawAll
	End If
End Sub

'gets or sets the TextColor property
Public Sub getTextColor As Int
	Return mTextColor
End Sub

Public Sub setTextColor(TextColor As Int)
	mTextColor = TextColor
	If xBase.IsInitialized Then
		DrawAll
	End If
End Sub

'gets or sets the HighlightTextColor property
Public Sub getHighlightTextColor As Int
	Return mHighlightTextColor
End Sub

Public Sub setHighlightTextColor(HighlightTextColor As Int)
	mHighlightTextColor = HighlightTextColor
	If xBase.IsInitialized Then
		DrawAll
	End If
End Sub

'gets or sets the SnapMode property
'possible values:
'AFTERMOVE	the knob snaps only when the knob is released, default value.
'ALLWAYS		the knob snaps during moving.
'NEVER			the knob doesn't snap at all, it displays the current position.  
Public Sub getSnapMode As String
	Return mSnapMode
End Sub

Public Sub setSnapMode(SnapMode As String)
	If SnapMode = "AFTERMOVE" Or SnapMode = "ALLWAYS" Or SnapMode = "NEVER" Then
		mSnapMode = SnapMode
		If xBase.IsInitialized Then
			DrawAll
		End If
	Else
		Log("Wrong SnapMode property value: only 'AFTERMOVE', 'ALLWAYS' and 'NEVER' are authorized")
	End If
End Sub

'gets or sets the CustomKnob property
'CustomKnob with a Bitmap. 
'The file must be in File.DirAsset
Public Sub getCustomKnob As Boolean
	Return mCustomKnob
End Sub

Public Sub setCustomKnob(CustomKnob As Boolean)
	mCustomKnob = CustomKnob
End Sub

'gets or sets the CustomKnobFileName property
'CustomKnob Bitmap file name. 
'Valid only with CustomKnob = True. 
'The file must be in File.DirAsset
Public Sub getCustomKnobFileName As String
	Return mCustomKnobFileName
End Sub

Public Sub setCustomKnobFileName(CustomKnobFileName As String)
	mCustomKnobFileName = CustomKnobFileName
End Sub

'sets or gets the value propertycof of the knob
Public Sub getValue As Int
	Return mValue
End Sub

Public Sub setValue(Value As Int)
	mValue = Value
	If mSnapMode <> "NEVER" Then
		mIndex = GetIndexValue(mValue)
		mValue = mScaleMinValue + mIndex * mScaleDivision
	Else
		DrawValue(mValue)
	End If
	mAngle = (mValue - mScaleMinValue) / mScale + mBeginAngle
	DrawKnob(mAngle)
End Sub
	
	'sets or gets the Visible propertycof the knob
Public Sub getVisible As Boolean
	Return xBase.Visible
End Sub

Public Sub setVisible(Visible As Boolean)
	xBase.Visible = Visible
End Sub

Private Sub MeasureTextWidth(Text As String, Font1 As B4XFont) As Int
	Private rct As B4XRect
	rct = xcvsBase.MeasureText(Text, Font1)
	Return rct.Width
End Sub

Private Sub MeasureTextHeight(Text As String, Font1 As B4XFont) As Int
	Private rct As B4XRect
	rct = xcvsBase.MeasureText(Text, Font1)
	Return rct.Height
End Sub